/*****************************************************************************
 *                          MODULE: SAMPLE1.C                                *
 *                                                                           *
 *  This sample program will sequentially turn on all bits in port a and then*
 *  seqencially turn them off.  Each time it sets a new bit, both port a and *
 *  port b are read and the data displayed.  This demonstrates how to read  *
 *  and write to a port, and to use the read back function of the 8255 chip. *
 *  If the port a pins are jumpered to the port b pins, then a board test    *
 *  program results, with port b being used to verify what has been written  *
 *  to port a.  The program will use port 0 of cards with mulitple 8255's.   *
 *                                                                           *
 *  LAST MODIFICATION: 2/5/98                                                *
 *                                                                           *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <conio.h>
#include <dos.h>

/* these are the port address offsets for reading and control of the IOD */
unsigned int PortA;
unsigned int PortB;
unsigned int PortC;
unsigned int PortControl;

/* this typedef allow use to duplicate the boolean type as used in pascal */

typedef enum {FALSE,TRUE} boolean;

/*****************************************************************************
 *  FUNCTION: signon -- local routine                                        *
 *                                                                           *
 *   PURPOSE: To display an initial sign on message on the screen.           *
 *                                                                           *
 *     INPUT: None.                                                          *
 *                                                                           *
 *    OUTPUT: None.                                                          *
 *                                                                           *
 *****************************************************************************/
void signon(void)
{
#define L1 " D I O  C A R D  S A M P L E  P R O G R A M "
#define L3 "Connect a loopback cable from PORT A to PORT B of PPI0."
#define L4 " PORT A OUTPUT     PORT A INPUT      PORT B INPUT "
#define L5 " -------------     ------------      ------------ "
#define L6 " PRESS ANY KEY TO EXIT PROGRAM "
#define L7 " PRESS ANY KEY TO CONTINUE"

   int	index;

   /* print the string display */
   clrscr();

   gotoxy(21,1);
   printf("%s",L1);
   gotoxy(10,4);
   printf("%s", L3);
   gotoxy(27,10);
   printf("%s", L7);
   getch ();
   gotoxy(17,6);
   printf("%s",L4);
   gotoxy(17,7);
   printf("%s",L5);
   gotoxy(27,10);
   printf("%s",L6);
} /* end signon */

/*****************************************************************************
 *  FUNCTION: write_port_data -- local routine                               *
 *                                                                           *
 *   PURPOSE: Displays the current port values on the screen.                *
 *                                                                           *
 *     INPUT: current: array with the port reading to display.               *
 *                                                                           *
 *****************************************************************************/
void write_port_data(unsigned char current[3])
{
unsigned      x,y,index1,index;
unsigned char value;

   x = 20;    /* x coordinate to write value to screen */
   y = 8;
   for (index = 0;index <= 2;index++)       /* for each array member */
      {
      value = current[index];
      for (index1 = 0;index1 <= 7;index1++) /* for each bit in array member */
         {
         gotoxy(x,y);
         if (value % 2) putchar('1');
         else putchar('0');
         value = value >> 1;                /* roll next diaplay bit */
         x++;
         }
      x+= 10;
      }
}  /* end write_port_data */

/******************************************************************************
Function    AskForBaseAddress
Parameters: OldOne
Returns:    Returns base address of Com

Purpose:    This function allows the user to input the address of the
	         card.
*****************************************************************************/


unsigned AskForBaseAddress(unsigned int OldOne)
{
	char msg[7];
	int NewOne = 0, Success = 0, Dummy;
	int AddrInputPosX, AddrInputPosY;

	puts("\nPlease enter the Base Address for your card (in hex)");
	printf("or press ENTER for %X.\n>", OldOne);
	AddrInputPosX = wherex(); AddrInputPosY = wherey();
	do {
      gotoxy(AddrInputPosX, AddrInputPosY);
      clreol();
      msg[0] = 5; msg[1] = 0;
	  cgets(msg);
      sscanf(msg + 2, "%x", &NewOne);
      Success = 1;
      Dummy = NewOne;
		if (msg[1] == 0) {
        gotoxy(AddrInputPosX, AddrInputPosY);
        printf("%X", OldOne);
        Success = 1;
        Dummy = OldOne;
      }
	} while(!Success);
	return (Dummy);
} /* end of AskForBaseAddress */


/*****************************************************************************
 *  FUNCTION: main -- external routine                                       *
 *                                                                           *
 *     INPUT: None.                                                          *
 *                                                                           *
 *****************************************************************************/
void main()
{
   unsigned char           current[3];
   boolean			shift_left;

   clrscr();
   puts("                      SAMPLE 1.C : DIO READ/WRITE                    ");
   puts("");
   puts("This sample program will sequentially turn on all bits in port a and then");
   puts("sequencially turn them off.  Each time it sets a new bit, both port a and ");
   puts("port b are read and the data displayed.  This demonstrates how to read ");
   puts("and write to a port, and to use the read back function of the 8255 chip.");

   gotoxy(1,8);
   PortA = AskForBaseAddress(0x2A0);

   PortB = PortA + 1;
   PortC = PortA + 2;
   PortControl = PortA + 3;
   clrscr();
   printf("\n\n\nBoard Configuration:\n\n");
   printf(" -- Base Address is %X in hex\n",PortA);
   printf(" -- All remaining jumper settings are irrelevant\n\n\n\n\n");
   printf("Press a key to continue");
   getch();
   signon();                          /* print the start up message         */
   outportb(PortControl,0x8b);        /* send the control byte              */
   current[0] = 0;
   shift_left = TRUE;
   while(!kbhit())                    /* loop until key pressed             */
      {
      outportb(PortA,current[0]);     /* write value to port a              */

      delay(10);
      current[1] = inportb(PortA);    /* read back what we wrote to port A  */
      current[2] = inportb(PortB);    /* read the data on port B            */
      write_port_data(current);       /* write new data to screen           */
      delay(700);

      /* compute value to turn on/off nesxt bit in line */
      if (current[0] == 0) shift_left = TRUE;
      if (current[0] == 255) shift_left = FALSE;
      if (shift_left) current[0] = (current[0] << 1) + 1;
      else current[0] = (current[0] - 1) >> 1;
      };
   clrscr();
} /* end main program */


